#! /usr/local/bin/ruby -T
##############################################################################
#
# FILE:		gateway.rb
# DESCRIPTION:	Web Gateway
# DATE:		Tue, May 23 2000
# AUTHOR:	Kouichi ABE (WALL)
# E-MAIL:	kouichi@MysticWALL.COM
# REVISION: 	$Id: gateway.rb,v 1.7 2000/05/30 16:25:43 kouichi Exp $
#
##############################################################################


require 'socket'
require 'cgi'

# Gateway Class
class Gateway
  H_LAST  = "Last-Modified"
  H_CTYPE = "Content-Type"

  def initialize(url)
    @host = "localhost"
    @port = 80
    @path = "/"
    @base = $1 if %r!(.*/)?! =~ url
    if url =~ %r!http://(.*?)(?::(\d+))?(/.*)!
      @host = $1
      @port = Integer($2) if $2
      @path = $3
    end
    @tmp  = "/tmp/file" + $$.to_s
    @sd	  = nil
    @fd	  = nil
  end

  def send_request
    @sd = TCPSocket::open(@host, @port)
    fd  = File::open(@tmp, "w")
    @sd.sync
    @sd.print "GET ",@path," HTTP/1.1\r\nHost: ",@host,"\r\n\r\n"
    fd.sync
    fd.write @sd.read
    fd.close
  end

  def recv_response
    # get response header
    fd	   = File::open(@tmp, "r")
    header = Hash.new
    while fd.gets
      if /([-\w]+):(.*)[\r\n]+$/i
	header[$1] = $2
	next
      end
      break if /^\s+$/
    end

    print "#{H_LAST}: #{header[H_LAST]}\r\n" if header.has_key?(H_LAST)
    print "#{H_CTYPE}: #{header[H_CTYPE]}\r\n\r\n"

    # get response entity
    no_base = true
    while fd.gets
      no_base = false if %r!<base\s+href>!oi
      if %r!</head>!oi && no_base
	print %Q!<base href="#{@base}">\n</head>\n!
	next
      end
      print
    end
#    File.unlink(@tmp)
  end
end

def gateway(cgi)
  url	= cgi['url'].to_s if cgi.has_key?('url')
  if url.empty? || (url.size == 7 && url == "http://")
    url = cgi['resv'].to_s
  end

  gw = Gateway.new(url)
  gw.send_request
  gw.recv_response
end

def print_menu(cgi)
  title = "Web Gateway"

  cgi.out {
    cgi.html {
      cgi.head { cgi.title{title} } +
      cgi.body {
	cgi.h1 {title} +
	cgi.form("post", "#{cgi.script_name}") {
	  cgi.hidden("action", "access") +
	  "Menu&nbsp;" +
	  cgi.popup_menu("resv",
			["http://www.impress.co.jp/",
			 "impress Group Information"],
			["http://japan.cnet.com/", "Japan CNET.com"],
			["http://satellite.nikkei.co.jp/", "Хͥå"],
			["http://www.hotwired.co.jp/", "HotWired Japan"],
			["http://www.yahoo.co.jp/", "Yahoo! Japan"]) +		
	  cgi.br +
	  "URL&nbsp;" +
	  cgi.text_field("url", "http://") +
	  cgi.br +
	  cgi.submit("Go!")
	}
      }
    }
  }
end

cgi = CGI.new("html4")
case cgi.params['action'].to_s
  when "access"
    gateway(cgi)
  else
    print_menu(cgi)
end

