/*****************************************************************************
 *
 * FILE:	functions.h
 * DESCRIPTION:	functions declaration file for wyvern sources
 * DATE:	Mon, Sep 27 1999
 * AUTHOR:	Kouichi ABE (WALL) / 
 * E-MAIL:	kouichi@MysticWALL.COM
 * URL:		http://www.MysticWALL.COM/cover.html
 * COPYRIGHT:	(C) 1999-2003 졿Kouichi ABE (WALL), All rights reserved.
 * LICENSE:
 *   Copyright (C) 1999-2003 Kouichi ABE (WALL) <kouichi@MysticWALL.COM>,
 *   All rights reserved.
 *  
 *   Redistribution and use in source and binary forms, with or without
 *   modification, are permitted provided that the following conditions
 *   are met:
 *
 *   1. Redistributions of source code must retain the above copyright
 *      notice, this list of conditions and the following disclaimer.
 *
 *   2. Redistributions in binary form must reproduce the above copyright
 *      notice, this list of conditions and the following disclaimer in
 *      the documentation and/or other materials provided with the
 *      distribution.
 *
 *   THIS SOFTWARE IS PROVIDED BY KOUICHI ABE (WALL) ``AS IS'' AND ANY
 *   EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 *   IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 *   PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL KOUICHI ABE (WALL) BE
 *   LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 *   CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 *   SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
 *   BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 *   WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
 *   OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN
 *   IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * $Id: functions.h,v 1.51.2.26 2003/08/18 12:57:45 kouichi Exp $
 *
 *****************************************************************************/

#ifndef	_functions_h
#define	_functions_h


/*
 *	alias.c
 */
extern void	aliasEntry(Alias **, const String, const String, alias_t);
extern int	translateAlias(Message *);


/*
 *	cond.c
 */
extern int	conditionalRequestChecker(Message *);


/*
 *	config.c
 */
extern void	readConfigFile(void);


/*
 *	daemon.c
 */
extern void	detach(void);
extern void	storePID(void);


/*
 *	date.c
 */
extern String	getTimeOfRFC822(time_t, int);
extern time_t	convertDate(String);


/*
 *	error.c
 */
extern void	errorDocEntry(ErrorDoc **, const int, const String);
extern void	errorDocChecker(Message *);
extern int	errorMessageSender(Thread *);


/*
 *	file.c
 */
extern StrList *	indexesEntry(String);
extern int		translateDocRoot(Message *);
extern int		translatedPathChecker(Message *);
extern void		fileMapping(Message *);


/*
 *	get.c
 */
void	openRequestFile(Thread *);


/*
 *	headers.c
 */
extern Headers *	requestHeaderEntry(Headers **, String);
extern Headers *	responseHeaderEntry(Headers **,
					    const String, const String);
extern String		headerLookup(Headers *, const String);
extern int		parseHeader(Message *);


/*
 *	hook.c
 */
extern int	wvRunHeaderParser(Message *);
extern int	wvRunAccessChecker(Message *);
extern int	wvRunTranslateURI(Message *);
extern int	wvRunCheckUserID(Message *);
extern int	wvRunAuthChecker(Message *);
extern int	wvRunTypeChecker(Message *);
extern int	wvRunFixups(Message *, hook_ordering_t);
extern int	wvRunHandler(Message *);
extern int	wvRunFilter(Message *);
extern int	wvRunLogger(Message *);
extern void	wvHookHeaderParser(int (*)(Message *), hook_ordering_t);
extern void	wvHookAccessChecker(int (*)(Message *), hook_ordering_t);
extern void	wvHookTranslateURI(int (*)(Message *), hook_ordering_t);
extern void	wvHookCheckUserID(int (*)(Message *), hook_ordering_t);
extern void	wvHookAuthChecker(int (*)(Message *), hook_ordering_t);
extern void	wvHookTypeChecker(int (*)(Message *), hook_ordering_t);
extern void	wvHookFixups(int (*)(Message *), hook_ordering_t);
extern void	wvHookHandler(int (*)(Message *), hook_ordering_t, const String);
extern void	wvHookFilter(int (*)(Message *), hook_ordering_t, const String);
extern void	wvHookLogger(int (*)(Message *), hook_ordering_t);


/*
 *	io.c
 */
extern int	nread(Thread *, String, size_t);
extern int	readline(Thread *, String, size_t);
extern int	iosFlush(Thread *);
extern int	iosWrite(Thread *, const char *, size_t);
extern int	iosPuts(Thread *, String);
extern int	iosPrintf(Thread *, const char *, ...);


/*
 *	log.c
 */
extern void	openLogFiles(void);
extern void	closeLogFiles(void);
extern void	closeLog(VirtualHost *);
extern int	defaultLogWriter(Message *);
extern void	die(const char *, ...);
extern void	wvSyslog(log_t, const char *, ...);


/*
 *	mime.c
 */
extern void	mimeTypeEntry(const String, const String);
extern int	defaultMimeTypeChecker(Message *);


/*
 *	module.c
 */
extern int	moduleLoad(const String);
extern int	moduleUnload(const String);
extern void	moduleUnloadVulnerable(void);
extern void	moduleInitializer(void);
extern void	moduleDelete(void);


/*
 *	network.c
 */
extern void	socketEntry(const String, const int, socket_t);
extern void	openChannels(void);
extern int	setNonBlocking(const int);
extern void	closeChannels(void);
extern String	getServerName(void);


/*
 *	request.c
 */
extern void	httpdMain(void);
extern int	parseURI(Message *);


/*
 *	response.c
 */
extern void	responseTimeout(int);
extern void	responseHandler(Thread *);


/*
 *	security.c
 */
extern void	enforceSecureLevel(void);
extern void	dropPrivilegeEternally(void);
extern int	dropPrivilege(void);
extern int	raisePrivilege(void);
extern int	safePathChecker(Message *, const String);
extern String	sanitizeURI(const String);
extern int	wvSafeOpen(const String, const int, struct stat *, bool_t);
extern FILE *	wvSafeFOpen(const String, const String);


/*
 *	signal.c
 */
extern void	signalTrap(void);


#if	USE_SSL
/*
 *	ssl.c
 */
extern SSL_CTX *	sslOpenChannel(void);
extern int		sslAccept(Thread *);
#endif	/* USE_SSL */


/*
 *	thread.c
 */
extern void	threadPoolInit(void);
extern Thread *	threadNew(int);
extern void	threadDetach(Thread *);
extern Thread *	threadActives(void);
extern int	threadSweep(void);
extern void	threadClear(Thread *);


/*
 *	user.c
 */
extern int	translateUserDir(Message *);
extern void	userCacheInit(void);


/*
 *	util.c
 */
extern int	wvSkipLine(const String);
extern String	wvChomp(String);
extern int	wvClose(int);
extern int	wvNoMemoryError(Message *);
extern int	wvStrcmp(const String, const String);
extern int	wvStrncmp(const String, const String, size_t);
extern int	wvStrcasecmp(const String, const String);
extern int	wvStrncasecmp(const String, const String, size_t);
extern String	wvStrncpy(String, const String, size_t);
extern String	wvStrdup(const String);
extern String	wvStrndup(const String, size_t);
extern void *	wvMalloc(size_t);
extern void *	wvCalloc(size_t, size_t);
#ifndef	HAVE_USLEEP
extern int	usleep(unsigned int);
#endif	/* HAVE_USLEEP */
#ifndef	HAVE_MKSTEMP
extern int	mkstemp(char *);
#endif	/* HAVE_MKSTEMP */


/*
 *	vhost.c
 */
extern VirtualHost *	vhostNew(const String);
extern void		vhostDelete(void);
extern int		vhostSwitch(Message *);


/*
 *	wyvern.c
 */
extern void	quit(void);


#endif	/* _functions_h */
