/*****************************************************************************
 *
 * FILE:	module.h
 * DESCRIPTION:	module header file for wyvern
 * DATE:	Sat, Sep 16 2000
 * AUTHOR:	Kouichi ABE (WALL) / 
 * E-MAIL:	kouichi@MysticWALL.COM
 * URL:		http://www.MysticWALL.COM/cover.html
 * COPYRIGHT:	(C) 2000-2003 졿Kouichi ABE (WALL), All rights reserved.
 * LICENSE:
 *   Copyright (C) 2000-2003 Kouichi ABE (WALL) <kouichi@MysticWALL.COM>,
 *   All rights reserved.
 *  
 *   Redistribution and use in source and binary forms, with or without
 *   modification, are permitted provided that the following conditions
 *   are met:
 *
 *   1. Redistributions of source code must retain the above copyright
 *      notice, this list of conditions and the following disclaimer.
 *
 *   2. Redistributions in binary form must reproduce the above copyright
 *      notice, this list of conditions and the following disclaimer in
 *      the documentation and/or other materials provided with the
 *      distribution.
 *
 *   THIS SOFTWARE IS PROVIDED BY KOUICHI ABE (WALL) ``AS IS'' AND ANY
 *   EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 *   IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 *   PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL KOUICHI ABE (WALL) BE
 *   LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 *   CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 *   SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
 *   BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 *   WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
 *   OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN
 *   IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * $Id: module.h,v 1.15.2.9 2003/01/27 09:38:34 kouichi Exp $
 *
 *****************************************************************************/

#ifndef	_module_h
#define	_module_h

#ifndef	MODULE_MAGIC_NUMBER_MAJOR
#define	MODULE_MAGIC_NUMBER_MAJOR	20021011
#endif	/* MODULE_MAGIC_NUMBER_MAJOR */
#define	MODULE_MAGIC_NUMBER_MINOR	0	/* 0...n */

#define	RET_NG		(-1)
#define	RET_OK		(0)
#define	RET_CONT	(1)
#define	RET_WAIT	(2)
#define	RET_EMB		(3)


typedef enum {
  HOOK_INITIAL	= -1,	/* run this hook first, before ANYTHING */
  HOOK_FIRST	= 0,	/* run this hook first */
  HOOK_MIDDLE	= 1,	/* run this hook somewhere */
  HOOK_LAST	= 2,	/* run this hook after every other hook */
  HOOK_FINAL	= 3	/* run this hook last, after EVERYTHING */
} hook_ordering_t;

typedef enum {
  SAFETY_STANDARD,
  SAFETY_VULNERABLE
} safety_t;

typedef struct _Module {	/* module structure */
  const int		version;	/* API version */
  const safety_t	safety;		/* module safety */
  const char *		name;		/* module name */
  const char *		info;		/* description such as version etc.. */
  const char *		author;		/* module creator */
  void *		handle;		/* module handle */
  const void		(*init)(void);	/* called when it's loaded */
  const void		(*finish)(void);/* called when it's unloaded */
  const void		(*register_hooks)(void);
} Module;

#endif	/* _module_h */
