/*****************************************************************************
 *
 * FILE:	status.h
 * DESCRIPTION:	response status code
 * DATE:	Sun, Oct  7 2001
 * AUTHOR:	Kouichi ABE (WALL) / 
 * E-MAIL:	kouichi@MysticWALL.COM
 * URL:		http://www.MysticWALL.COM/cover.html
 * COPYRIGHT:	(C) 2001-2003 졿Kouichi ABE (WALL), All rights reserved.
 * LICENSE:
 *   Copyright (C) 2001-2003 Kouichi ABE (WALL) <kouichi@MysticWALL.COM>,
 *   All rights reserved.
 *  
 *   Redistribution and use in source and binary forms, with or without
 *   modification, are permitted provided that the following conditions
 *   are met:
 *
 *   1. Redistributions of source code must retain the above copyright
 *      notice, this list of conditions and the following disclaimer.
 *
 *   2. Redistributions in binary form must reproduce the above copyright
 *      notice, this list of conditions and the following disclaimer in
 *      the documentation and/or other materials provided with the
 *      distribution.
 *
 *   THIS SOFTWARE IS PROVIDED BY KOUICHI ABE (WALL) ``AS IS'' AND ANY
 *   EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 *   IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 *   PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL KOUICHI ABE (WALL) BE
 *   LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 *   CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 *   SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
 *   BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 *   WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
 *   OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN
 *   IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * $Id: status.h,v 1.3.6.3 2003/01/27 09:38:35 kouichi Exp $
 *
 *****************************************************************************/

#ifndef	_status_h
#define	_status_h

/*
 *	Status Code Definitions
 */
typedef enum {
  STATUS_NONE = -1,
#define	STATUS_CODE_1XX	(0)
  /* Informational 1xx */
  STATUS_CONTINUE = 1,			/* 100 */
  STATUS_SWITCHING_PROTOCOLS,		/* 101 */
  /* Successful 2xx */
#define	STATUS_CODE_2XX	(2)
  STATUS_REQUEST_OK,			/* 200 */
  STATUS_CREATED,			/* 201 */
  STATUS_ACCEPTED,			/* 202 */
  STATUS_NON_AUTH_INFO,			/* 203 */
  STATUS_NO_CONTENT,			/* 204 */
  STATUS_RESET_CONTENT,			/* 205 */
  STATUS_PARTIAL_CONTENT,		/* 206 */
  /* Redirection 3xx */
#define	STATUS_CODE_3XX	(9)
  STATUS_MULTIPLE_CHOICES,		/* 300 */
  STATUS_MOVED_PERMANENTLY,		/* 301 */
  STATUS_FOUND,				/* 302 */
  STATUS_SEE_OTHER,			/* 303 */
  STATUS_NOT_MODIFIED,			/* 304 */
  STATUS_USE_PROXY,			/* 305 */
  STATUS_TEMPORARY_REDIRECT,		/* 307 */
  /* Client Error 4xx */
#define	STATUS_CODE_4XX	(16)
  STATUS_BAD_REQUEST,			/* 400 */
  STATUS_UNAUTHORIZED,			/* 401 */
  STATUS_PAYMENT_REQUIRED,		/* 402 */
  STATUS_FORBIDDEN,			/* 403 */
  STATUS_NOT_FOUND,			/* 404 */
  STATUS_METHOD_NOT_ALLOWED,		/* 405 */
  STATUS_NOT_ACCEPTABLE,		/* 406 */
  STATUS_PROXY_AUTH_REQUIRED,		/* 407 */
  STATUS_REQUEST_TIMEOUT,		/* 408 */
  STATUS_CONFLICT,			/* 409 */
  STATUS_GONE,				/* 410 */
  STATUS_LENGTH_REQUIRED,		/* 411 */
  STATUS_PRECONDITION_FAILED,		/* 412 */
  STATUS_REQUEST_ENTITY_TOO_LARGE,	/* 413 */
  STATUS_REQUEST_URI_TOO_LONG,		/* 414 */
  STATUS_UNSUPPORTED_MEDIA_TYPE,	/* 415 */
  STATUS_REQUESTED_RANGE_NOT_SATISFIABLE,	/* 416 */
  STATUS_EXPECTATION_FAILED,		/* 417 */
  /* Server Error 5xx */
#define	STATUS_CODE_5XX	(34)
  STATUS_INTERNAL_SERVER_ERROR,		/* 500 */
  STATUS_NOT_IMPLEMENTED,		/* 501 */
  STATUS_BAD_GATEWAY,			/* 502 */
  STATUS_SERVICE_UNAVAILABLE,		/* 503 */
  STATUS_GATEWAY_TIMEOUT,		/* 504 */
  STATUS_HTTP_VERSION_NOT_SUPPORTED	/* 505 */
} status_t;


/*
 *	Table of response code and phrase
 */
typedef struct _ResponseStatus {
  int		code;	/* response status code */
  char *	phrase;	/* response phrase */
} ResponseStatus;

#ifdef	MAIN_MODULE
const ResponseStatus	responseStatusTable[] = {
  {0, "dummy"},
  /* Informational 1xx */
  {100, "Continue"},				/* 100 */
  {101, "Switching Protocols"},			/* 101 */
  /* Successful 2xx */
  {200, "OK"},					/* 200 */
  {201, "Created"},				/* 201 */
  {202, "Accepted"},				/* 202 */
  {203, "Non-Authoritative Information"},	/* 203 */
  {204, "No Content"},				/* 204 */
  {205, "Reset Content"},			/* 205 */
  {206, "Partial Content"},			/* 206 */
  /* Redirection 3xx */
  {300, "Multiple Choices"},			/* 300 */
  {301, "Moved Permanently"},			/* 301 */
  {302, "Found"},				/* 302 */
  {303, "See Other"},				/* 303 */
  {304, "Not Modified"},			/* 304 */
  {305, "Use Proxy"},				/* 305 */
  {307, "Temporary Redirect"},			/* 307 */
  /* Client Error 4xx */
  {400, "Bad Request"},				/* 400 */
  {401, "Unauthorized"},			/* 401 */
  {402, "Payment Required"},			/* 402 */
  {403, "Forbidden"},				/* 403 */
  {404, "Not Found"},				/* 404 */
  {405, "Method Not Allowed"},			/* 405 */
  {406, "Not Acceptable"},			/* 406 */
  {407, "Proxy Authentication Required"},	/* 407 */
  {408, "Request Timeout"},			/* 408 */
  {409, "Conflict"},				/* 409 */
  {410, "Gone"},				/* 410 */
  {411, "Length Required"},			/* 411 */
  {412, "Precondition Failed"},			/* 412 */
  {413, "Request Entity Too Large"},		/* 413 */
  {414, "Request-URI Too Long"},		/* 414 */
  {415, "Unsupported Media Type"},		/* 415 */
  {416, "Requested Range Not Satisfiable"},	/* 416 */
  {417, "Expectation Failed"},			/* 417 */
  /* Server Error 5xx */
  {500, "Internal Server Error"},		/* 500 */
  {501, "Not Implemented"},			/* 501 */
  {502, "Bad Gateway"},				/* 502 */
  {503, "Service Unavailable"},			/* 503 */
  {504, "Gateway Timeout"},			/* 504 */
  {505, "HTTP Version Not Supported"}		/* 505 */
};
#else
extern ResponseStatus	responseStatusTable[];
#endif	/* MAIN_MODULE */

#endif	/* _status_h */
