/*****************************************************************************
 *
 * FILE:	wyvern.h
 * DESCRIPTION:	global header file for wyvern
 * DATE:	Thu, Aug 26 1999
 * AUTHOR:	Kouichi ABE (WALL) / 
 * E-MAIL:	kouichi@MysticWALL.COM
 * URL:		http://www.MysticWALL.COM/cover.html
 * COPYRIGHT:	(C) 1999-2003 졿Kouichi ABE (WALL), All rights reserved.
 * LICENSE:
 *   Copyright (C) 1999-2003 Kouichi ABE (WALL) <kouichi@MysticWALL.COM>,
 *   All rights reserved.
 *  
 *   Redistribution and use in source and binary forms, with or without
 *   modification, are permitted provided that the following conditions
 *   are met:
 *
 *   1. Redistributions of source code must retain the above copyright
 *      notice, this list of conditions and the following disclaimer.
 *
 *   2. Redistributions in binary form must reproduce the above copyright
 *      notice, this list of conditions and the following disclaimer in
 *      the documentation and/or other materials provided with the
 *      distribution.
 *
 *   THIS SOFTWARE IS PROVIDED BY KOUICHI ABE (WALL) ``AS IS'' AND ANY
 *   EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 *   IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 *   PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL KOUICHI ABE (WALL) BE
 *   LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 *   CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 *   SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
 *   BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 *   WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
 *   OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN
 *   IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * $Id: wyvern.h,v 1.74.2.31 2003/08/18 12:57:45 kouichi Exp $
 *
 *****************************************************************************/

#ifndef	_wyvern_h
#define	_wyvern_h

#ifdef	MAIN_MODULE
#define	EXTERN
#else
#define	EXTERN extern
#endif	/* MAIN_MODULE */

#include <stdio.h>
#include <sys/types.h>
#include <sys/param.h>
#include <sys/stat.h>
#include <sys/socket.h>
#include <netdb.h>
#if	USE_SSL
#include <openssl/ssl.h>
#endif	/* USE_SSL */
#include <sys/time.h>
#include "status.h"
#include "module.h"
#include "version.h"


/*****************************************************************************
 *
 *	definition of common macros
 *
 *****************************************************************************/

#define	BUF_SIZE	(BUFSIZ*4)

#define	CR		'\015'
#define	LF		'\012'
#define	CRLF		"\015\012"
#define	EOL		'\0'
#define	SPC		'\040'
#define	TAB		'\011'

#ifndef	MAX
#define	MAX(a,b)	(((a)>(b))?(a):(b))
#endif	/* MAX */
#ifndef	MIN
#define	MIN(a,b)	(((a)<(b))?(a):(b))
#endif	/* MIN */

#if	HAVE_STRTOL
#define	atoi(x)		(int)strtol((x),(char **)NULL,10)
#endif	/* HAVE_STRTOL */

#ifndef	wvFree
#define	wvFree(x)	if((x)!=NULL)free((x))
#endif	/* wvFree */


/*
 *	size and length
 */
#define	RESPONSE_BUFSIZE	(2048)
#define	SOCKET_BUFSIZE		(4096)

#define	ALIAS_TABLE_SIZE	(11)	/* Redirect/Alias/ScriptAlias */
#define	ERRDOC_TABLE_SIZE	(5)	/* customizable error documents */
#define	MIME_TABLE_SIZE		(79)	/* mime.types file */
#define	USER_TABLE_SIZE		(13)	/* user's home directory cache */

#define	MAX_TIMEOUT		(60*60*24)	/* 24 hour */
#define	MAX_CGI_ENVNO		(34+1)	/* max number of CGI environment */
#define	MAX_URI_LEN		(256)	/* maximum URI length */
#define	MAX_FILENAME_LEN	(80)
#define	MAX_NAME_LEN		(16)
#define	MAX_FILE_SIZE		(3*1024*1024)	/* file resource limit: 3MB */
#if	MAXPATHLEN
#define	MAX_PATH_LEN		MAXPATHLEN
#else
#define	MAX_PATH_LEN		(512)
#endif	/* MAXPATHLEN */
#if	MAXHOSTNAMELEN
#define	MAX_HOSTNAME_LEN	MAXHOSTNAMELEN
#else
#define	MAX_HOSTNAME_LEN	(256)
#endif	/* MAXHOSTNAMELEN */

/*
 *	users/groups
 */
#define	UID_ROOT	(0)	/* super user id in /etc/passwd */
#define	GID_WHEEL	(0)	/* super user's group id in /etc/passwd */
#define	UID_WWW		(80)	/* www user id in /etc/passwd */
#define	GID_WWW		(80)	/* www id in /etc/passwd */
#define	UID_NOBODY	(65534)	/* nobody user id in /etc/passwd */
#define	GID_NOGROUP	(65533)	/* nogroup id in /etc/passwd */

/*
 *	default value
 */
#define	DEF_PORT	"80"			/* Port */
#define	DEF_SECURE_PORT	"443"			/* SecurePort via SSL */
#define	DEF_MIN_THREAD	(2)			/* Minimum Thread */
#define	DEF_MAX_THREAD	(8)			/* MaxThread */
#define	DEF_MIN_REQUEST	(2)			/* Minimum Request */
#define	DEF_MAX_REQUEST	(8)			/* MaxRequest */
#define	DEF_USER	UID_WWW			/* User */
#define	DEF_GROUP	GID_WWW			/* Group */
#ifndef	SERVER_ROOT
#define	DEF_SERVER_ROOT	"/usr/local/wyvern"
#else
#define	DEF_SERVER_ROOT	SERVER_ROOT
#endif	/* SERVER_ROOT */
#define	DEF_SECURE_LEVEL	(-1)		/* SecureLevel */
#define	DEF_SECURITY_LOG	DEF_LOG_DIR "/security_log" /* SecurityLog */
#define	DEF_PID_FILE	DEF_LOG_DIR "/wyvern.pid"	/* PidFile */
#define	DEF_ERROR_LOG	DEF_LOG_DIR "/error_log"	/* ErrorLog */
#define	DEF_ACCESS_LOG	DEF_LOG_DIR "/access_log"	/* AccessLog */
#define	DEF_TIMEOUT	(30)			/* Timeout (sec.) */
#define	DEF_KA_TIMEOUT	(10)			/* KeepAliveTimeout (sec.) */
#define	DEF_KA_MAX	(15)			/* KeepAliveMax */
#define	DEF_DOC_ROOT	DEF_SERVER_ROOT "/htdocs"	/* DocumentRoot */
#define	DEF_USERDIR	"public_html"		/* UserDir */
#define	DEF_DIR_INDEX	"index.html"		/* DirectoryIndexes */
#define	DEF_SYMLINKS	(0)			/* FollowSymLinks (0:off 1:on)*/
#define	DEF_DEFTYPE	"text/plain"		/* DefaultType */
#define	DEF_CONFIG	DEF_SERVER_ROOT "/conf/wyvern.conf"
#define	DEF_VHOSTNAME	"default"

/*
 *	directories
 */
#define	DEF_LOG_DIR	DEF_SERVER_ROOT "/logs"
#define	DEF_TMP_DIR	DEF_SERVER_ROOT "/tmp"
#define	DEF_CACHE_DIR	DEF_SERVER_ROOT "/cache"
#define	ETC_DIR		DEF_SERVER_ROOT "/etc"

/*
 */
//#define	HTTP_VERSION	"HTTP/1.0"
#define	CGI_VERSION	"CGI/1.1"
#define	ALLOW_METHODS	"HEAD,GET,POST"
#define	RANGE_UNIT	"bytes"
#define	TYPE_TEXT_HTML	"text/html"
/**/
#define	DEFAULT_PATH		"/bin:/usr/bin:/usr/local/bin"
#define	ANY_MAGIC_TYPE		""
#define	HTML_MAGIC_TYPE		"text/html"
#define	PLAIN_MAGIC_TYPE	"text/plain"
#define	CGI_MAGIC_TYPE		"application/x-httpd-cgi"
#define	SSI_MAGIC_TYPE		"text/x-server-parsed-html"
#define	ECGI_MAGIC_TYPE		"application/x-httpd-embedded-cgi"
#define	SUEXEC_MAGIC_TYPE	"application/x-httpd-suexec"
#define	DIR_MAGIC_TYPE		"httpd/unix-directory"

/*
 * secure level
 */
#define	SECURE_LEVEL0	(0)
#define	SECURE_LEVEL1	(1)
#define	SECURE_LEVEL2	(2)
#define	SECURE_LEVEL3	(3)
#define	SECURE_LEVEL4	(4)
#define	SECURE_LEVEL5	(5)
#define	SECURE_LEVEL6	(6)
#define	SECURE_LEVEL7	(7)

/*
 * timezone type
 */
#define	TIMEZONE_GMT	(1)
#define	TIMEZONE_LOCAL	(0)	/* now TIMEZONE_LOCAL is JST */

/*
 */
#define	POLL_IN		(1)
#define	POLL_OUT	(4)

/*
 */
#define	CGI_ERROR_STATUS	(99)
#define	IO_USLEEP_TIME		(1000)	/* 0.01 seconds */
#define	SSL_USLEEP_TIME		(100000)/* 0.1 seconds */
#define	TILDE_OFFSET		(2)	/* offset of '~' */
#define	TIMEOUT_INFINITY	(-1)


/* error messages */
#define	ERR_OUT_OF_MEMORY	"No enough memory\n"

/*****************************************************************************/

#if	USE_SSL
#define	SSL_ERROR	(-1)
#define	SSL_OK		(0)
#define	SSL_RETRY	(1)
#endif	/* USE_SSL */


/*****************************************************************************
 *
 *	definition of common types and macros
 *
 *****************************************************************************/

#ifndef	_STRING_T
#define	_STRING_T
typedef char *	String;	/* standard string type */
#endif	/* _STRING_T */

#ifndef	_BOOL_T
#define	_BOOL_T
#ifdef	TRUE
#undef	TRUE
#endif	/* TRUE */
#ifdef	FALSE
#undef	FALSE
#endif	/* FALSE */
typedef enum {
  FALSE = 0,
  TRUE	= 1
} bool_t;
#endif	/* _BOOL_T */


/*****************************************************************************
 *
 *	definition of structures
 *
 *****************************************************************************/

typedef enum {	/* log type */
  LOG_FATAL = 1,
  LOG_EMERG,	/* system is unusable */
  LOG_ALERT,	/* action must be taken immediately */
  LOG_CRIT,	/* critical conditions */
  LOG_ERR,	/* error conditions */
  LOG_WARNING,	/* warning conditions */
  LOG_NOTICE,	/* normal but significant condition */
  LOG_INFO,	/* informational */
  LOG_DEBUG,	/* debug-level messages */
  LOG_SECURITY = 16	/* security messages (special) */
} log_t;

typedef struct _StrList {	/* simple string list */
  String		text;
  struct _StrList *	next;
} StrList;

typedef struct _Log {	/* logfile structure */
  String		path;	/* path for log file */
  FILE *		fp;	/* file pointer for log */
} Log;

typedef enum {
  ED_STRING,	/* such as "Not found..." */
  ED_FILE,	/* such as "/error/404.html" */
  ED_URL	/* such as "http://www.example.com/cgi-bin/404.cgi" */
} errdoc_t;

typedef struct _ErrorDoc {
  int			code;	/* response status code */
  String		text;	/* document to response */
  errdoc_t		type;	/* error document type */
  struct _ErrorDoc *	next;
} ErrorDoc;

typedef enum {	/* alias type */
  A_REDIRECT,	/* Redirect */
  A_ALIAS,	/* Alias */
  A_SCRIPT	/* ScriptAlias */
} alias_t;

typedef struct _Alias {	/* alias structure */
  alias_t		type;		/* alias type */
  String		fakeName;	/* URI path to file */
  String		realName;	/* actual path to file */
  size_t		fakeLen;	/* strlen of fakeName */
  size_t		realLen;	/* strlen of realName */
  struct _Alias	*	next;
} Alias;

typedef enum {
  SOC_SECURE,	/* via SSL */
  SOC_UNSECURE	/* standard connection */
} socket_t;

typedef struct _ListenAddr {
  String		ipaddr;	/* ip address */
  String		port;	/* port number */
  socket_t		type;	/* socket type */
  struct _ListenAddr *	next;
} ListenAddr;

typedef struct _Socket {
#if	USE_SSL
  SSL_CTX *		ssl_ctx;/* pointer of one in VirtualHost */
  SSL *			ssl;	/* SSL connection */
#endif	/* USE_SSL */
  int			sd;	/* socket descriptor */
  socket_t		type;	/* socket type */
  int			port;	/* port number */
  short			events;	/* socket event */
  short			revents;/* socket event after select() */
  struct addrinfo	ainfo;	/* address information for socket */
  struct _Socket *	next;
} Socket;

typedef struct _VirtualHost {	/* virtual host structure */
  String	serverName;	/* ServerName */
  String	serverAdmin;	/* ServerAdmin */
  Log		errorLog;	/* ErrorLog */
  Log		accessLog;	/* AccessLog */
  unsigned int	timeout;	/* Timeout */
  /**/
  String	docRoot;	/* DocumentRoot */
  String	userDir;	/* UserDir */
  StrList *	dirIndexes;	/* DirectoryIndexes */
  bool_t	autoIndexing;	/* AutoIndexing */
  bool_t	symlinks;	/* FollowSymLinks */
  String	defaultType;	/* DefaultType */
  /**/
  ErrorDoc *	errorDocTable;
  Alias *	aliasTable;
  /**/
#if	0	//USE_SSL
  SSL_CTX *	ssl_ctx;
  String	sslCertificateFile;	/* SSLCertificateFile */
  String	sslCertificateKeyFile;	/* SSLCertificateKeyFile */
#endif	/* USE_SSL */
  /**/
  struct _VirtualHost *	next;
} VirtualHost;

/*
 * wyvern HTTP daemon structure (for wyvern.conf)
 */
typedef struct _httpd {
  /* config */
  ListenAddr *	listen;		/* Listen */
  int		maxThread;	/* MaxThread */
  int		maxRequest;	/* MaxRequest */
  uid_t		user;		/* User */
  gid_t		group;		/* Group */
  String	serverRoot;	/* ServerRoot */
  int		secureLevel;	/* SecureLevel */
  Log		securityLog;	/* SecurityLog */
  String	pidFile;	/* PidFile */
  pid_t		pid;		/* Wyvern PID */
  /**/
  VirtualHost *	vhostTable;	/* virtual hosts manager (linked list) */
  VirtualHost *	host;		/* original server host */
  VirtualHost *	vhost;		/* current virtual host */
  /* internal */
  bool_t	shutdown;	/* shutdown flag */
  /**/
  Socket *	sockets;
  int		maxsoc;		/* maximum socket descriptor's number */
  /**/
  struct {
    int		signo;
    bool_t	sigexit;	/* SIGTERM/SIGINT/SIGQUIT */
    bool_t	sighup;		/* SIGHUP */
  } trap;
  /**/
  time_t	startuptime;	/* timestamp when wyvern booted */
  /**/
  String	tmpDir;		/* temporary working directory */
  String	cacheDir;	/* CacheDir */
  String	configFile;	/* wyvern.conf path */
} Httpd;

/*****************************************************************************/

typedef enum {  /* method type */
  M_NONE,
  M_HEAD,	/* See RFC2616 Section 9.4 (HEAD request) */
  M_GET,	/* See RFC2616 Section 9.3 (GET  request) */
  M_POST,	/* See RFC2616 Section 9.5 (POST request) */
#if	0
  M_OPTIONS,	/* See RFC2616 Section 9.2 (OPTIONS request) */
  M_PUT,	/* See RFC2616 Section 9.6 (PUT request) */
  M_DELETE,	/* See RFC2616 Section 9.7 (DELETE request) */
  M_TRACE,	/* See RFC2616 Section 9.8 (TRACE request) */
  M_CONNECT,	/* See RFC2616 Section 9.9 (CONNECT request */
  M_MOVE,	/* extension method of Netscape Communicator */
  M_SEARCH,	/* Wyvern original method (SEARCH request) */
#endif
  M_UNKNOWN
} method_t;
#ifdef	MAIN_MODULE
String	 	methodName[] = {"-", "HEAD", "GET", "POST", NULL};
#else
extern String 	methodName[];
#endif	/* MAIN_MODULE */

typedef enum {	/* HTTP version */
  HTTP_00,	/* unknown version */
  HTTP_09,	/* version 0.9 */
  HTTP_10,	/* version 1.0 */
  HTTP_11	/* version 1.1 */
} http_t;
#ifdef	MAIN_MODULE
String		httpVer[] = {NULL, "HTTP/0.9", "HTTP/1.0", "HTTP/1.1"};
#else
extern String 	httpVer[];
#endif	/* MAIN_MODULE */

typedef enum {	/* KeepAlive state type */
  KA_DISABLED,
  KA_OK,
  KA_CLOSE,
  KA_CONTINUE
} keepalive_t;

typedef struct _Range {
  int	first;	/* first byte position */
  int	last;	/* last byte position */
} Range;

typedef struct _Headers {
  String		name;
  String		value;
  struct _Headers *	next;
} Headers;

/* client connection structure */
typedef struct _Connection {
  /* client's socket */
  Socket	sock;

  /* client's hostname and IP address */
  struct {	/* used by getnameinfo(3) */
    String	port;	/* remote port */
    String	ip;	/* remote IP address */
    String	name;	/* remote hostname */
    struct sockaddr_storage	addr;
  } client;

  /* keepalive */
  struct {
    keepalive_t		status;
    unsigned int	timeout;	/* KeepAliveTimeout */
    unsigned int	max;		/* KeepAliveMax */
  } keepalive;
} Connection;

typedef enum {
  MSG_NONE = -1,
  MSG_READING,
  MSG_READY,
  MSG_NEGOTIATION,
  MSG_PROCESSING,
  MSG_WRITING_READY,
  MSG_WRITING,
  MSG_ERROR,
  MSG_WAITING,
  MSG_IDLING,
  MSG_LOGGING,
  MSG_CLOSING
} message_t;

/*
 * client message structure
 */
typedef struct _Message {
  /* message status */
  message_t	status;

  /* current virtual host */
  VirtualHost *	vhost;

  /* client connection information */
  Connection *	conn;

  /* last accepted request/response timestamp */
  time_t	accesstime;

  /* Request */
  struct {
    method_t	method;			/* HTTP method */
    char	uri[MAX_URI_LEN];	/* request URI */
    http_t	http;			/* HTTP version */
    Headers *	headers;		/* request header fields */
    String	path_info;		/* for PATH_INFO */
    String	query_string;		/* for QUERY_STRING */
    int		post_fd;		/* file descriptor for POST method */
    String	post_tmp;		/* temporary file for POST method */
    /**/
    String	host;			/* Host: */
    size_t	content_length;		/* Content-Length: */
    String	x_forwarded_for;	/* X-Forwarded-For: */
  } request;

  /* Response */
  struct {
    status_t	status;		/* HTTP response status code */
    Headers *	headers;	/* response header fields */
    size_t	content_length;	/* Content-Length */
    String	error_mesg;	/* error message pointer */
  } response;

  /* content */
  struct {
    int		fd;	/* file descriptor */
    String	data;	/* pointer for data file */
    String	curp;	/* current pointer in data */
    size_t	rest;	/* rest size of content */
    String	path;	/* actual path of data file */
    String	mime;	/* MIME type */
    struct stat	status;	/* file status */
    Range	range;	/* range unit for partial request */
  } file;

  /* user */
  struct {
    String	indexDir;	/* path of user directory (i.e. public_html) */
    String	homeDir;	/* user home directory */
    String	name;		/* owner of UserDir */
    String	authName;	/* authenticated username */
    String	identName;	/* identification username */
  } user;

  /* cgi */
  struct {
    pid_t	pid;		/* pid of CGI program */
    int		envno;		/* the number of CGI environments */
    String *	env;		/* environments for CGI program */
    String	status;		/* CGI has own response status line */
    bool_t	nph;		/* Non-Parsed Header */
  } cgi;

  struct _Message *	next;
} Message;

typedef enum {	/* free -> active -> detached -> free */
  TH_FREE,
  TH_ACTIVE,
  TH_TERMINATED
} thread_t;

typedef struct _Thread {
#if	DEBUG
  int		no;
#endif	/* DEBUG */
  /* thread status */
  thread_t	status;

  /* timestamp */
  struct {
    struct timeval	accepted;	/* accepted new connection */
    struct timeval	last;		/* last access time */
    struct timeval	current;	/* current time */
  } time;

  /* client connection information */
  Connection	conn;

  /* I/O buffer */
  struct {
    bool_t	update;		/* flag for buffer status */
    int		total;		/* sum of character in buf */
    int		next;		/* next character in buf */
    char	buf[BUF_SIZE];	/* I/O buffer */
  } ios;

  /* client request */
  Message	mqueue;	/* request queue for pipelining */
  Message *	msg;	/* current request */

  struct _Thread *	next;
} Thread;

/*****************************************************************************/

EXTERN Httpd *	httpd;	/* daemon entity */

#endif	/* _wyvern_h */
